function sprintf(s, args) {
    var bits = s.split('%');
    var out = bits[0];
    var re = /^([ds])(.*)$/;
    for (var i=1; i<bits.length; i++) {
        p = re.exec(bits[i]);
        if (!p || args[i-1]==null) continue;
        if (p[1] == 'd') {
            out += parseInt(args[i-1], 10);
        } else if (p[1] == 's') {
            out += args[i-1];
        }
        out += p[2];
    }
    return out;
}

/**
 * Returns localized string for the current languague and requested key (see errors.js)
 *
 * If no match is found or invalid kay is specified returns string passed as the
 * second parameter.
 *
 * Also supports parametrized messages in the sprintf manner:
 *  %d is replaced by number specified as the call argument
 *  %s is replaced by string specified as the call argument
 *
 * Please NOTE: Replacement works only for defined messages and doesn't work
 *   for the ones passed as second parameter.
 *
 * Example:
 *  define key:message pair somewhere (for ex. errors.js):
 *    lang['existingKey'] = 'Message with %d number and %s string parameter';
 *  get the message:
 *    getText('exisitngKey', 'Default message', 15, 'custom')
 *  will produce following message:
 *    'Message with 15 number and custom string parameter'
 *
 *  However if the 'existingKey' will not exist in the lang[] associative array
 *  function call will return string 'Default message' either.
 *
 */
function getText(key, string) {
    if (typeof(lang) != 'undefined' && lang[key]) {
        text = lang[key];
        var replacements = new Array();
        for (var i = 1; i < arguments.length - 1; i++) {
            replacements[i-1] = arguments[i + 1];
        }
        if (typeof(replacements) != 'undefined' && replacements.length > 0) {
            text = sprintf(text, replacements);
        }
        return text;
    }
    return string;
}

