var base = frontEndBaseUrl;
var droid = !!(navigator.userAgent.match(/Android/));
var ios = !!(navigator.userAgent.match(/iPhone/) || navigator.userAgent.match(/iPod/) || navigator.userAgent.match(/iPad/));
var droidMajorVersion = navigator.userAgent.match(/Android \d/ig) + '';
if (droid) {
    droidMajorVersion = parseInt(droidMajorVersion.match(/\d/) + '');
}
var tapEvent = (ios || droidMajorVersion >= 4)? 'vclick' : 'click';
var mainScriptLoaded = false;
var networkTimer;
var isCurrentAppVersion = true;
var inDevMode = false;

$(function() {
    $('#cantAccessLink').attr('href', base + 'acctResetPswrd');
    $('#acctLeanTest').attr('href', base + 'noaccess');

    if (ios) {
        $('body').addClass('ios');
    }
    else if (droid) {
        $('body').addClass('droid');
    }

    $(document).bind('deviceready', deviceReady);

    if (supportsLocalStorage()) {
        rememberIdBehavior();
    } else {
        $('#rememberId').hide();
        $('#userid').css('width', '100%');
    }
});

function deviceReady() {
    $("#login a").bind(tapEvent, handleLoginLinks);
    $("#login form").bind("submit", handleLoginForms);
    loadRetinaNativeImages();

    checkNetwork();
    loadMainApplicationScript();

    $(document).bind('resume', function() {
        setTimeout(function() {
            if (!mainScriptLoaded) {
                // we have interrupted the ajax request for mih.js...
                reloadPage();
            }else {
                // When the app resumes after having been run in the backgbround we can't assume that we have the latest version of the app
                isCurrentAppVersion = false;
                checkNetwork();
            }
        }, 0);
    });
}

/*
 * Ensure that we are always running the latest app
 * Called by networkConnection() which is called by checkNetwork()
*/
function checkAppVersion() {
    if (isCurrentAppVersion || !mainScriptLoaded) {
        return;
    }
    var p = new Date().getTime(); // prevent caching
    $.get(base + 'resources/currentAppVersion.txt?q=' + p, function(data) {
        var currentApp = $.parseJSON(data);

        if (currentApp && currentApp.version && appVersion && currentApp.version != appVersion) {
            if (inDevMode) {
                alert(currentApp.version + ' ' + appVersion);
            }
            // Force reload to get the latest app. This condition will most likely be triggered when the user is on the sign in page
            appVersion = data.version;
            if ($.mobile.activePage.attr('id') === 'login') {
                reloadPage();
            }else {
                doSignOut();
            }
        } else {
            isCurrentAppVersion = true;
        }
    });
}

/* login page link event handler that messages user in case of network connection failure */
function handleLoginLinks(e) {
    if (!mainScriptLoaded && !$(e.target).closest('#rememberId').length) {
        removeTapHilite();
        showAppError();
        return false;
    }
}
/* login page form submit event handler that messages user in case of network connection failure */
function handleLoginForms(e) {
    var form = $(e.target), id = form.attr('id');
    if (!mainScriptLoaded) {
        hideKeyboard();
        scrollUp();
        showAppError();
        return false;
    }
}

// Show application error if the network connection is OK
function showAppError() {
    if (!$("#login .networkError").is(":visible")) {
        setTimeout("$('#loginForm').find('.instructions').hide();" +
            "displayNativeError('The application is currently unavailable. Please try again in a few moments.');", 0);
    }
}

// load the main application JS file only when network connection is assured
function loadMainApplicationScript() {
    if (mainScriptLoaded) {
        return;
    }

    $.ajax({
        url: base + "resources/js/mih.js",
        dataType: "script",
        cache: false,
        timeout: 10000,
        success: function() {
            var p = new Date().getTime(); // ensure that CSS isn't cached

            mainScriptLoaded = true;
            $.getScript(base + "resources/js/error.js");
            $('<link rel="stylesheet" />').attr("href", base + "resources/final/css/mobile.css?q=" + p).appendTo($("head"));
            displayNativeError('noerror', false);
            $('#login .instructions').show();

            // unbind login page handlers
            $("#login a").unbind(tapEvent, handleLoginLinks);
            $("#login form").unbind("submit", handleLoginForms);
        },
        error: function() {
            showAppError();
            loadMainApplicationScript();
            checkNetwork();
        }
    });
}

/*
    PhoneGap Only - before executing event handlers for link click/tap and form submit, check the network connection.
    If there isn't one cancel the event and display an error message.
*/
function checkNetwork() {
    var $form, $container, $networkError;

    if (!networkConnection()) {
        $container = $.mobile.activePage.find(".ui-content");
        $networkError = $container.find(".networkError");
        $.mobile.activePage.find('.errorMsg').hide();
        hideKeyboard();
        scrollUp();
        if (!$networkError.length) {
            $form = $container.find("form");
            if ($form.length) {
                $container = $form;
            }
            $networkError = $('<div class="networkError">The network connection is offline.</div>').prependTo($container);
        }
        removeTapHilite();
        $networkError.removeClass("delayedFlash");
        if ($networkError.is(":visible")) {
            $networkError.addClass("delayedFlash");
        }else {
            $networkError.show();
        }
        if (!networkTimer) {
            networkTimer = setInterval(networkConnection, 400);
        }
        if (typeof resumeTap === 'function') {
            resumeTap();
        }
        return false;
    }else {
        return true;
    }
}
function networkConnection() {
    var connection = navigator.network.connection.type;

    if (connection === "none" || connection === "unknown") {
        return false;
    }else {
        if (networkTimer) {
            clearInterval(networkTimer);
            networkTimer = null;
        }
        checkAppVersion();
        $.mobile.activePage.find(".networkError").hide();
        if (!$("#login .errorMsg").is(":visible")) {
            $('#loginForm .instructions').show();
        }
        return true;
    }
}

// hide URL bar
function scrollUp() {
    $.mobile.silentScroll(droid? 1 : 0);
}

// hide the virtual keyboard
function hideKeyboard() {
    setTimeout(function() {
        try {
            $('#hiddenInput').focus();
        }catch(e) {}
    }, 1);
}

function removeTapHilite(context) {
    $(".tapactive", context || $.mobile.activePage).removeClass("tapactive");
}

/*
    Captures the user ID when the login form is submitted but doesn't save it unless login is successful (home page show)
    A little messy because mih.js handles login and clearing of the user ID field
*/
function rememberIdBehavior() {
    var userID = localStorage.getItem("userid");

    if (userID) {
        $('#userid').val(userID);
        $('#rememberId').addClass('checked');
    }
    $('#rememberId').bind(tapEvent, function() {
        var $link = $(this);

        $link.toggleClass('checked');
        if (!$link.hasClass('checked')) {
            // remove user ID from local storage immediately if unchecked (don't wait for sign in)
            localStorage.setItem('userid', '');
        }
        removeTapHilite();
        try {
            setTimeout(resumeTap, 100);
        }catch(e) {}
    });
    $('#loginForm').submit(function() {
        userID = $('#rememberId').hasClass('checked')? $.trim($('#userid').val()) : '';
    });
    $(document).bind('pageshow', saveUserId);
    $(document).bind('pageshow', function(e) {
        var page = $(e.target), id = page.attr('id');
        doGoogleAnalytics(id);
    });

    function doGoogleAnalytics(pageName) {
        ga('set', 'page', '/#' + pageName);
        ga('set', 'title', document.title);
        ga('send', 'pageview', {});
    }

    function saveUserId(e) {
        var page = e.target;

        if (page.id === 'home') {
            localStorage.setItem('userid', userID);
            $(document).unbind('pageshow', saveUserId);
        }
    }
}
function supportsLocalStorage() {
    try {
        return 'localStorage' in window && window['localStorage'] !== null;
    }catch (e) {
        return false;
    }
}

/************************
 * PhoneGap Plugins
 ************************/

window.phoneCall = function(num) {
    num += "";
    num = num.replace(/\)/, "-");
num = num.replace(/[^\d\-]/g, "");
    cordova.exec(
        function(msg) {
            // success
        },
        function(err) {
            // fail
            alert(err);
        },
        "PhoneCall", "dial", [num]
    );
};

function displayNativeError(message, flag) {
    if (flag == undefined || flag) {
        removeTapHilite();
        var formErrorDiv = $('div.ui-page-active .errorMsg');
        if (!formErrorDiv || formErrorDiv.length == 0) {
            // No error DIV, we need to create it

            // Some pages have padding on their content DIV.  In order for the
            // error DIV to not be "floating" because of the padding, we
            // remove it by adding margin:0 .
            var pad = $('div.ui-page-active div[data-role=content]')
                .css('padding-top');
            var marginShift = '';
            if (pad == '0px' || pad == '0') {
                // Need to take off the margin shift
                marginShift = ' style="margin: 0;"';
            }

            $('div.ui-page-active  div[data-role=content]')
                    .prepend('<div class="errorMsg"'
                        + marginShift
                        + '></div>');
            formErrorDiv = $('div.ui-page-active .errorMsg');
        }
        formErrorDiv.html(message).show();
        scrollUp();
    }
    else {
        $('div.ui-page-active .errorMsg').hide().remove();
    }
}

function loadRetinaNativeImages() {
    var nativeImages = ["resources/images/button.png", "resources/images/icon_calendar_24_2x.png", "resources/images/icon_medications_24_2x.png", "resources/images/icon_ask_24_2x.png", "resources/images/icon_billPay_24_2x.png", "resources/images/icon_messages_48_2x.png"];
    for (var i = 0, n = nativeImages.length; i < n; i++) {
        (new Image()).src = nativeImages[i];
    }
}

function showMap(address) {
  window.plugins.webintent.startActivity({
    action: WebIntent.ACTION_VIEW,
    url: 'geo:0,0?q=' + address
  }, function () {}, function () {
    alert('Failed to open URL via Android Intent');
  });
}

function reloadPage()
{
    window.location.hash = '';
    if (droid) {
        window.location = 'file:///android_asset/www/index.html';
    }
    else {
        window.location.reload();
    }
}
